/*
synchronous version

*/
#include <avr/io.h>
#include <avr/interrupt.h>
//#include "t26Serial.h"


#define OUTPUT             1
#define INPUT              0

#define syncBit            3
#define syncPort           PINB


#define IsHigh(BIT, PORT)    ((PORT & (1<<BIT)) != 0)
#define IsLow(BIT, PORT)     ((PORT & (1<<BIT)) == 0)
#define SetBit(BIT, PORT)     PORT |= (1<<BIT)
#define ClearBit(BIT, PORT)   PORT &= ~(1<<BIT)

#define ABS(a)                ((a) < 0 ? -(a) : (a))
#define SIGN(x)               (x)==0?0:(x)>0?1:-1
#define NOP()                 asm volatile ("nop"::)


/*****************************| VARIABLES |********************************/
 
volatile unsigned int  AdcValues[10];
 
/************************| FUNCTION DECLARATIONS |*************************/
 
void AnalogInit (void);
void sendByte(unsigned char byte);
void syncSerialInit(void);
unsigned char dobyte(char data);



/*
 [0][c3][c2][c1][c0][v2][v1][v0]
 [1][v9][v8][v7][v6][v5][v4][v3]
 
 c0-c3 are channel
 v0-v9 are value
*/

#define PackBits(V,P) (((V << 5) & 0xFF00)|(V & 0x07)|0x8000|(P<<3))


/****************************| CODE... |***********************************/

int main (void) {
 
  unsigned int temp;
  unsigned char i;
  unsigned int buffer;

  // set up directions 
    

  DDRA = (INPUT << PA0 | INPUT << PA1 | INPUT << PA2 | INPUT << PA3 | INPUT << PA4 | INPUT << PA5 | INPUT << PA6 | INPUT << PA7);
  DDRB = (INPUT << PB0 | OUTPUT << PB1 | INPUT << PB2 | INPUT << PB3 | INPUT << PB4 | INPUT << PB5 | INPUT << PB6 );

  AnalogInit();
 // initSerial(103); // 9600 baud at 8Mhz
  syncSerialInit();
  
  // turn interrupts on
  sei();
  

  while(1){
   
    for (i = 0; i < 10; i++) {
    
    // sendChar(' ');
     
     temp = AdcValues[i];       //snapshot         
     buffer = PackBits(temp, i);
     sendByte( buffer & 0xFF );
     sendByte( buffer >> 8 );
     
    // USART_printhex(buffer & 0xFF);
    // USART_printhex(buffer >> 8);
          
    }
    
   // sendChar('\r');
   // sendChar('\n');  
  }

}




//------------------------| FUNCTIONS |------------------------


unsigned char dobyte(char data) {

  static unsigned char state = 0;
  static unsigned int command;
  static unsigned int  argument;

  if (state == 0) {
    if ((data & 0x80) == 0) {
      state    = 1;
      command  = (data >> 3);
      argument = (argument & 0xFFF8) | (data & 0x07); // glue in its 0 through 2
    } 
  } else {
    state = 0;
    if ((data & 0x80) != 0) {
      argument = (argument & 0x0007) | ((data & 0x7F) << 3); //glue in bits 3 through 9
      return servoCmd(command, argument);   
    }
  } 

  return 0;
}




void AnalogInit (void) {

  int i;

  // Activate ADC with Prescaler 
  ADCSR =   1 << ADEN  |
            1 << ADSC  | /* start a converstion, irq will take from here */
            0 << ADIF  |
            1 << ADIE  | /* enable interrupt */
            1 << ADPS2 | /* prescalar divide by 64 */
            1 << ADPS1 |
            0 << ADPS0 ;
                        
            
  for (i = 0; i < 10; AdcValues[i++] = 0);
  
}


ISR(ADC_vect) {
  unsigned char i;
  
  i = ADMUX & 31;                         // what channel we on?
  AdcValues[i] = ADC;                     // save value
  i = ((i + 1) % 10);                     // next value (0 thru 9, we will leave channel 10 cause its the reset line)
  ADMUX = i ;                             // select channel
  ADCSR |= _BV(ADSC);                     // start next conversion
  return;
}


void syncSerialInit(void) {
  USICR |= (1<<USIWM0) | (1<<USICS1);
}


void sendByte(unsigned char byte) {

  // wait for sync to be high
  while (IsLow(syncBit, syncPort));
  // wait for sync to be low
  while (IsHigh(syncBit, syncPort));
  // write byte 
  USIDR = byte;

}









